﻿<#
.SYNOPSIS
   Converts Green Browser Group files to HTML bookmarks (Mozilla/Chrome)
.DESCRIPTION
   Reads all ANSI .cgp files in a folder according to encoding specified, saves
   to temp UTF-8 file, then reads file and outputs to Mozilla/Chrome compatible
   bookmark format.
.PARAMETER <paramName>
   None
.EXAMPLE
   PS> CGP2HTML.ps1
#>

Function Get-Folder {
  Add-Type -AssemblyName System.Windows.Forms
  $FolderBrowser = New-Object System.Windows.Forms.FolderBrowserDialog
  [void]$FolderBrowser.ShowDialog()
  Return $FolderBrowser.SelectedPath
}  

Function Convert-UTF {
  Param(
    [String]$SourceFile
  )
  $Buffer = Get-Content $SourceFile -Encoding byte
  
  $Encoding = [System.Text.Encoding]::GetEncoding("gb2312")
  $String = $Encoding.GetString($Buffer)
  Out-File -FilePath .\C2Htmp.txt -Encoding utf8 -InputObject $String
}

Function Write-Header {
  Param(
    [String]$OutFile,
    [String]$Folder
  )
  '<!DOCTYPE NETSCAPE-Bookmark-file-1>' | Out-File -Encoding utf8 -FilePath $OutFile -NoClobber
  '<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<TITLE>Bookmarks</TITLE>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<H1>Bookmarks</H1>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<DT><H3>{0} --- {1}</H3>' -f $Folder, (Get-Date) | Out-File -Encoding utf8 -FilePath $OutFile -Append
  '<DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
}

Function Out-Bookmark {
  Param(
    [Object]$File,
    [String]$Outfile
  )
  for($i=0; $i -lt $File.Length; $i++) {
    $tmpFile = $File[$i] | % { $_.BaseName }
    Write-Host "Converting: " $File[$i]
    Convert-UTF $File[$i]
    '<DT><H3>' + $tmpFile + '</H3></DT>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
    '<DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
    $Lines = (Get-Content ".\C2Htmp.txt")
    for($j=2; $j -lt $Lines.Length; $j+=2) {
      '<DT><A HREF="{0}">{1}</A>' -f $Lines[$j].Substring($Lines[$j].IndexOf("=") + 1), `
                                     $Lines[$j - 1].Substring($Lines[$j - 1].IndexOf("=") + 1) `
                                     | Out-File -Encoding utf8 -FilePath $OutFile -Append
    }
    '</DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
    Remove-Item '.\C2HTmp.txt'
  }
  '</DL><P>' | Out-File -Encoding utf8 -FilePath $OutFile -Append
}

If($PSVersionTable.PSVersion.Major -lt 4) {
  Write-Host '** Script requires at least Powershell V4 **'
} else {
  $sFolder = (Get-Folder)
  If($sFolder -eq '') {Exit}
  Write-Host '-- Conversion started --'
  $aFiles = (Get-ChildItem -Include *.cgp -Path ($sFolder + "\*"))
  If(($sFolder.LastIndexOf('\') + 1) -eq $sFolder.Length) {
  $sOutfile =  ($sFolder | % { $_.Substring(0, $_.LastIndexOf('\') + 1) }) `
                + 'CGP2HTML-' + ("{0:yyyy}" -f (Get-Date)) + ("{0:MM}" -f (Get-Date)) `
                + ("{0:dd}" -f (Get-Date)) + ('{0:hh}' -f (Get-Date)) `
                + ('{0:mm}' -f (Get-Date)) + ('{0:ss}' -f (Get-Date)) + '.html'
  } else {
  $sOutfile =  $sFolder + '\' + 'CGP2HTML-' + ("{0:yyyy}" -f (Get-Date)) `
                + ("{0:MM}" -f (Get-Date)) + ("{0:dd}" -f (Get-Date)) `
                + ('{0:hh}' -f (Get-Date)) + ('{0:mm}' -f (Get-Date)) `
                + ('{0:ss}' -f (Get-Date)) + '.html'
  }
  Write-Header $sOutfile $sFolder
  Out-Bookmark $aFiles $sOutfile
  Write-Host '-- Conversion finished --'
  Write-Host ''
  Write-Host 'Output file: ' $sOutfile -foregroundcolor "blue" -backgroundcolor "yellow"
}
Write-Host ''
Write-Host 'Press a key to exit ...'
cmd /c pause | out-null
